// project created on 06.08.2002 at 19:45
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace BubbleGame
{

class Tile
{		
	int x, y, targetX, targetY, colorIndex, ticker;	
	const int SIZE = 40;
	const int COLORS = 10;		
	int transparency = 255;
	
	bool marked, switching, fading, played;
	bool alive = true;
	bool settled = true;
	
	Color color;		
	Timer timer = new Timer();	
	System.Threading.Thread thread = null;
	string soundFile = "sound.wav";
	
	[DllImport("winmm.dll")]
	public static extern long PlaySound(String lpszName, long hModule, long dwFlags);
		
	public Color[] colors = new Color[COLORS]
	{
		Color.Red,
		Color.Blue,
		Color.Yellow,
		Color.DarkOliveGreen,
		Color.PaleVioletRed,
		Color.BurlyWood,
		Color.CornflowerBlue,
		Color.LightYellow,
		Color.MediumSeaGreen,
		Color.Orange
	};
	
	
	public Tile(int x, int y)
	{			
		this.x = targetX = x;
		this.y = targetY = y;		
		
		timer.Interval = 20;	
		timer.Tick += new EventHandler(Ticker);
		timer.Start();
	}
	
	void Ticker(object sender, EventArgs e)
	{			
		if(x != targetX || y != targetY)
		{
			ticker = 0;
			settled = false;
		}
		else
			ticker ++;
		
		if(ticker > 20)							
		{
			settled = true;	
			switching = false;
		}
		
		if(fading)
		{	
			if(transparency > 10)
				transparency -= 10;			
			else
				{
					alive = false;						
					timer.Dispose();
				}		
		}
		
		int SPEED;
		
		if(switching)
			SPEED = 4;
		else
			SPEED = 2;
		
		if(alive && fading == false)
		{
			if(x < targetX)	
				x += SPEED;
			else if(x > targetX)
				x -= SPEED;		
			
			if(y < targetY)	
				y += SPEED;	
			else if(y > targetY)
				y -= SPEED;			
		}
	}		
	
	public void SetColor(int col)
	{
		colorIndex = col;
		this.color = colors[colorIndex];		
	}
	
	public void Mark()
	{		
		marked = true;
	}
	
	public void Unmark()
	{
		marked = false;
	}
		
	//check if this tile is eastern neighbor of the passed tile
	public bool IsEastNeighbor(Tile t)
	{		
		if(t.alive == false || this.alive == false)
			return false;
		
		if(this.y == t.y && this.x == t.x + SIZE/* && this.settled == true && t.settled == true*/)
			return true;	
		
		return false;
	}
	
	public bool IsWestNeighbor(Tile t)
	{
		if(t.alive == false || this.alive == false)
			return false;
		
		if(this.y == t.y && this.x == t.x - SIZE)
			return true;	
		
		return false;
	}
	
	public bool IsSouthNeighbor(Tile t)
	{
		if(t.alive == false || this.alive == false)
			return false;
		
		if(this.x == t.x && this.y == t.y + SIZE/* && this.settled == true && t.settled == true*/)
			return true;
		
		return false;
	}
		
	public bool IsNorthNeighbor(Tile t)
	{
		if(t.alive == false || this.alive == false)
			return false;
		
		if(this.x == t.x && this.y == t.y - SIZE)
			return true;		
		
		return false;
	}
		
	public bool Fading
	{
		get{ return fading; }
		set{ fading = value; }
	}
	
	public bool Switching
	{
		get{ return switching; }
		set{ switching = value; }
	}
	
	public bool Alive
	{
		get{ return alive; }		
	}
	
	public bool Settled
	{
		get{ return settled; }		
	}
	
	public bool Marked
	{
		get{ return marked; }	
		set{ marked = value; }
	}
	
	public int X
	{
		get{ return x; }		
	}
	
	public int Y
	{
		get{ return y; }		
	}
	
	public bool MouseInside(Point p)
	{
		Rectangle r =  new Rectangle(x, y, SIZE, SIZE);
		
		if(r.Contains(p))
			return true;
		
		return false;	
	}
	
	//reports back where the mouse hovers	
	public int HoverOver(Point p)
	{
		Rectangle r1 = new Rectangle(x, y + 10, 15, 20);		//West
		Rectangle r2 = new Rectangle(x + 10, y, 20, 15);		//North
		Rectangle r3 = new Rectangle(x + 25, y + 10, 15, 20);	//East
		Rectangle r4 = new Rectangle(x + 10, y + 25, 20, 15);	//South
			
		if(r1.Contains(p))
			return 1;
		else if(r2.Contains(p))
			return 2;
		else if(r3.Contains(p))
			return 3;
		else if(r4.Contains(p))
			return 4;
		
		return 0;		
	}
	
	//check if some tile has the same color
	public bool SameColor(Tile t)
	{
		if(t.colorIndex == this.colorIndex)
			return true;		
		
		return false;
	}	
	
	public void Destroy()
	{		
		fading = true;
		
		if(played == false)
		{			
			PlaySoundInThread(soundFile);			
			played = true;
		}
	}
	
	public void MoveTo(int targetX, int targetY)
	{					
		this.targetX = targetX;				
		this.targetY = targetY;					
	}
	
	void PlayASound()
	{
		PlaySound(Application.StartupPath + "\\" + soundFile, 0, 0);	
		thread.Abort();
	}

	void PlaySoundInThread(string wavefile)
	{			
		if (thread != null)
			thread.Abort();
		
		thread = new System.Threading.Thread(new System.Threading.ThreadStart(PlayASound));
		thread.Start();					
	}
	
	public void Draw(Graphics g)  
	{							
		if(marked && alive && settled && fading == false)
			g.FillRectangle(new SolidBrush(Color.Plum), x, y, SIZE, SIZE);	
		
		SolidBrush semiTransBrush = new SolidBrush(Color.FromArgb(transparency, color.R, color.G, color.B));
		
		if(alive)
			g.FillEllipse(semiTransBrush, x, y, SIZE, SIZE);		
		
		if(marked && alive && settled && fading == false)
			g.DrawRectangle(new Pen(Color.White), x, y, SIZE, SIZE);	
	} 
}
}

